/********************************************************************************
 Author : Akshay Kumar B U (akshay.kumar@analog.com)
					India Applications Support Team 

 Date : January 2014

 File name : ADuCM360Driver.c

 Description : This executable file contains definitions of all the functions defined in the header file ADuCM360Driver.h

 Hardware plateform : ADuCM360 	+ AD5687
********************************************************************************/

#include <ADuCM360.h>
#include "ADuCM360Driver.h"
#include "AD5687.h"
#include "ClkLib.h"
#include "IntLib.h"
#include "DioLib.h"
#include "WdtLib.h"
#include "DioLib.h"


unsigned char InputBuf[10]={0x00,0x00,0x00};
unsigned char OutputBuf[10]={0x00,0x00,0x00};


// Delay Routine
void Delay(unsigned long int DelayTime)			  
{ 
		while(DelayTime>0)
	{ 
				DelayTime--;
	}
}

//Initialize ADuC360 for AD5687
void ADuCM360_Initialize(void)
{
	WdtCfg(T3CON_PRE_DIV1,T3CON_IRQ_EN,T3CON_PD_DIS);  // Disable Watchdog timer resets
  DioOen(pADI_GP1,0x3E);         // Set P1.1, P1.2,P1.3, P1.4 and P1.5 as an output  and P1.7 as an input pins.
	
 
  //Disable clock to unused peripherals
  ClkDis(CLKDIS_DISSPI0CLK|CLKDIS_DISSPI1CLK|CLKDIS_DISI2CCLK|CLKDIS_DISUARTCLK|CLKDIS_DISPWMCLK|CLKDIS_DIST0CLK|CLKDIS_DIST1CLK|CLKDIS_DISDACCLK|CLKDIS_DISDMACLK|CLKDIS_DISADCCLK);
  ClkCfg(CLK_CD0,CLK_HF,CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);            // Select CD0 for CPU clock - 16Mhz clock
	
}

/**
	@brief void SPIinitialize()
	- Generic 'SPIinitialize' the SPI pins with default levels
		@ no return value
**/
void SPIinitialize()
{
  //Initialize the serial interface pins
	ADuCM360GpioOutput(AD5687_SYNC, PIN_HIGH);
	ADuCM360GpioOutput(AD5687_SCLK, PIN_LOW);
	ADuCM360GpioOutput(AD5687_SDIN, PIN_LOW);
}

/**
	@brief void ADuCM360GpioOutput  (unsigned char  GpioNo, unsigned char OutputValue)
	- Configures the GPIO's as output based on pin assigned to the Controllers
	@param GpioNo : Pin number XY, X - Port number, Y - Pin Number
		@ - no return value
**/
void ADuCM360GpioOutput  (unsigned char  GpioNo, unsigned char OutputValue)
{
	unsigned char PortNo  =  GpioNo  & 0xF0;
	unsigned char PinNo  =  GpioNo  & 0x0F;
	switch(PortNo)
	{	
		case 0x00:
				if(OutputValue == PIN_HIGH)
					pADI_GP0->GPSET =  (1<< PinNo);
				else if(OutputValue == PIN_LOW)
					pADI_GP0->GPCLR =  (1<< PinNo);
				break;
				
		case 0x10:
				if(OutputValue == PIN_HIGH)
					pADI_GP1->GPSET =  (1<< PinNo);
				else if(OutputValue == PIN_LOW)
					pADI_GP1->GPCLR =  (1<< PinNo);
				break;
				
		case 0x20: 
				if(OutputValue == PIN_HIGH)
					pADI_GP2->GPSET =  (1<< PinNo);
				else if(OutputValue == PIN_LOW)
					pADI_GP2->GPCLR =  (1<< PinNo);
				break;
				
		default:
				break;
	}
	
}

/**
	@brief unsigned int ADuCM360GpioInput (unsigned char  GpioNo)
	- Configures the GPIO's as input based on pin assigned to the Controllers
	@param GpioNo : Pin number XY, X - Port number, Y - Pin Number
	@return Status
		Status - returns either high or low
**/
unsigned int ADuCM360GpioInput  (unsigned char  GpioNo)
{
	unsigned char PortNo  =  GpioNo  & 0xF0;
	unsigned char PinNo  =  GpioNo  & 0x0F;
  int inputbit = 0;
		
	switch(PortNo)
	{	
		case 0x00:
				if((1<< PinNo) == ((pADI_GP0->GPIN) & (1<< PinNo)))     // Returns the value at Dout/Rdy pin bit by bit.
					inputbit= 1;
				else
					inputbit = 0;
				break;
				
		case 0x10:
				if((1<< PinNo) == ((pADI_GP1->GPIN) & (1<< PinNo)))     // Returns the value at Dout/Rdy pin bit by bit.
					inputbit = 1;
				else
					inputbit = 0;
				break;
				
		case 0x20: 
				if((1<< PinNo) == ((pADI_GP2->GPIN) & (1<< PinNo)))     // Returns the value at Dout/Rdy pin bit by bit.
					inputbit = 1;
				else
					inputbit = 0;
				break;
				
		default:
				break;
	}
	
	return(inputbit);	
}


/**
	@brief void SpiReadWrite(unsigned char* InputBuffer, unsigned char* OutputBuffer, unsigned char NoOfByte)
	- Generic SPI Read and Write
		@param InBuf :8 bit output data array
           OutBuf: 8 bit input data array
		NoOfBytes : (0x00-0x20) Number of Bytes required to transfer
		@ no return value
**/
void SpiReadWrite(unsigned char *InBuf, unsigned char *OutBuf, unsigned char NoOfBytes)
{
int i,j;
unsigned int TempInput = 0x00;
unsigned int TempOutput = 0x00;	
unsigned int ReceivedBit = 0x00;

	ADuCM360GpioOutput  (AD5687_SYNC, PIN_LOW);
	Delay(100);
	
	for(i=0; i< NoOfBytes; i++)
	{
		TempOutput = *OutBuf;
		
		for(j=0; j<8; j++)
		{
				////////////WRITE DATA/////////////
			ADuCM360GpioOutput(AD5687_SCLK, PIN_HIGH);
			Delay(100);			
					
			
			if(0x80==(TempOutput & 0x80))
				{
					ADuCM360GpioOutput(AD5687_SDIN, PIN_HIGH);
				}
			else
				{
					ADuCM360GpioOutput(AD5687_SDIN, PIN_LOW);
				}
				
				TempOutput<<=1;
			 	Delay(100);
		
			////////////WRITE DATA ENDS/////////
			
			
			///////////READ DATA///////
				
			ADuCM360GpioOutput(AD5687_SCLK, PIN_LOW);
			Delay(100);	
			
			TempInput<<=1;	   
			ReceivedBit = ADuCM360GpioInput(AD5687_SDO);	
				if(0x1==ReceivedBit)
					{
						TempInput |= 1;
					}
				
			////////////READ DATA ENDS/////////
						
		}
		*InBuf = TempInput;
		OutBuf++;
		InBuf++;
			
	}
	
	ADuCM360GpioOutput (AD5687_SYNC, PIN_HIGH);
	Delay(1000);
	
}

/**
	@brief unsigned int WriteRegister(unsigned long int WriteData)
	- Write the data to the Register
		@param WriteData :{0x000000 - 0xFFFFFF}
		@ no return value
**/
void WriteRegister(unsigned long int WriteData)
{
	unsigned char Byte;
	
	for(Byte=0;Byte < NO_OF_BYTES;Byte++)
	{
		*(OutputBuf + Byte) = (WriteData>>(((NO_OF_BYTES-1)-Byte)*8)); //Writing the data to be written to 8 bit Output Buffer
		*(InputBuf + Byte) = 0x00;
	}
	SpiReadWrite( InputBuf, OutputBuf, NO_OF_BYTES); //Writes data to the specified register

}
